import torch, numpy as np
from torch import nn
from torch.utils.data import DataLoader, TensorDataset
from ..model.cosine_kernel import CosineKernel

def run_dummy_epoch() -> None:
    x = torch.randn(64, 16)  # fake data
    y = torch.randn(64, 16)
    loader = DataLoader(TensorDataset(x, y), batch_size=32)
    model = nn.Linear(16, 16)
    opt = torch.optim.AdamW(model.parameters(), lr=1e-3)
    kernel = CosineKernel()
    for xb, yb in loader:
        loss = 1.0 - torch.mean(torch.tensor([
            kernel.forward(a.numpy(), b.numpy()) for a, b in zip(xb, yb)
        ]))
        opt.zero_grad(); loss.backward(); opt.step()

if __name__ == "__main__":
    run_dummy_epoch()


import logging, sys
from structlog import wrap_logger

_logging = logging.getLogger("linker")
_logging.setLevel(logging.INFO)
_logging.addHandler(logging.StreamHandler(sys.stdout))
logger = wrap_logger(_logging)
__all__ = ["logger"]


from fastapi import FastAPI
from pydantic import BaseModel
import numpy as np
from ..extractor import extract
from ..model.cosine_kernel import CosineKernel

app = FastAPI(title="Linker Scaffold API")
kernel = CosineKernel()

class Query(BaseModel):
    text: str

@app.get("/ping")
def ping(): return {"pong": True}

@app.post("/link")
def link(q: Query):
    tuples = extract(q.text)
    vec = np.ones(16)  # placeholder fixed vector
    score = kernel.forward(vec, vec)
    return {"edges": [{"src": q.text, "dst": "dummy", "score": score, "tuples": tuples}]}


import numpy as np
from numpy import ndarray
from .kernel_base import LinkerKernel

class CosineKernel(LinkerKernel):
    def forward(self, vec_a: ndarray, vec_b: ndarray) -> float:
        return float(np.dot(vec_a, vec_b) / (np.linalg.norm(vec_a) * np.linalg.norm(vec_b) + 1e-9))